@@ -18,7 +18,7 @@ from pyntcloud import PyntCloud
 import ot
 from multipoles import MultipoleExpansion
 
-def get_border_dict(labels,states=None,radius=10,vdist=None,return_nnindex=True,return_nnvector=True,return_curvature=True,nn_states=None,scale=None,**border_args):
+def get_border_dict(labels,states=None,radius=10,state_ids=None,vdist=None,return_nnindex=True,return_nnvector=True,return_curvature=True,nn_states=None,scale=None,**border_args):
     """
     Computes the border properties of labeled regions in a segmented image.
 
@@ -90,6 +90,12 @@ def get_border_dict(labels,states=None,radius=10,vdist=None,return_nnindex=True,
     border_dict['scale']=scale
     if states is None:
         states=np.zeros(np.max(labels+1)).astype(int);states[0]=0;states[1:]=1
+    if state_ids is None:
+        nstates=np.max(states)+1
+    else:
+        nstates1=np.max(state_ids)+1
+        nstates2=np.max(states)+1
+        nstates=np.max([nstates1,nstates2])
     border=skimage.segmentation.find_boundaries(labels,mode='inner')
     ind=np.where(border)
     border_pts=np.array(ind).astype(float).T
@@ -129,10 +135,10 @@ def get_border_dict(labels,states=None,radius=10,vdist=None,return_nnindex=True,
         border_dict['nn_pts']=border_nn_pts
         border_dict['nn_inds']=border_nn_inds
         if nn_states is not None:
-            border_nn_pts_state=np.ones((border_pts.shape[0],np.max(states)+1,border_pts.shape[1]))*np.nan
-            border_nn_inds_state=np.ones((border_pts.shape[0],np.max(states)+1)).astype(int)*-1
+            border_nn_pts_state=np.ones((border_pts.shape[0],nstates,border_pts.shape[1]))*np.nan
+            border_nn_inds_state=np.ones((border_pts.shape[0],nstates)).astype(int)*-1
             #knn_states=[None]*(np.max(states)+1)
-            inds_states=[None]*(np.max(states)+1)
+            inds_states=[None]*(nstates)
             iset=np.where(np.isin(states,nn_states))[0]
             stateset=np.unique(states)
             for istate in stateset:
@@ -1399,7 +1405,7 @@ def get_flux_ligdist(vdist,cmean=1.,csigma=.5,center=True):
     fsigmas=np.abs(csigma*np.abs(vdist))
     return fmeans,fsigmas
 
-def get_border_properties(cell_labels,surfaces=None,cell_states=None,surface_states=None,cell_labels_next=None,cell_labels_prev=None,lin_next=None,lin_prev=None,vdist=None,border_scale=1.0,radius=2.0,order=1,return_border_properties_list=False):
+def get_border_properties(cell_labels,surfaces=None,cell_states=None,state_ids=None,surface_states=None,cell_labels_next=None,cell_labels_prev=None,lin_next=None,lin_prev=None,vdist=None,border_scale=1.0,radius=2.0,order=1,return_border_properties_list=False):
     """
     Calculate geometric and physical properties of cell borders, with optional tracking between frames 
     and surface contact details.
@@ -1499,7 +1505,10 @@ def get_border_properties(cell_labels,surfaces=None,cell_states=None,surface_sta
         cell_states=[1]*max_cellid
         cell_states=np.array(cell_states).astype(int)
         cell_states=np.insert(cell_states,0,0)
-    max_cell_state=np.max(cell_states)
+    if state_ids is None:
+        max_cell_state=np.max(cell_states)
+    else:
+        max_cell_state=np.max(np.setdiff1d(state_ids,surface_states))
     if surfaces is None:
         n_surf=0
     else:
@@ -1512,7 +1521,6 @@ def get_border_properties(cell_labels,surfaces=None,cell_states=None,surface_sta
             if np.intersect1d(cell_states,surface_states).size>0:
                 print('Conflicting labels between cell states and surface states provided')
         cell_states=np.append(cell_states,surface_states)
-        nn_states=cell_states.copy()
         #print(f'cell states {cell_states}')
         surface_overlay=surfaces[0].copy()
         for i_surf in range(n_surf):
@@ -1529,7 +1537,9 @@ def get_border_properties(cell_labels,surfaces=None,cell_states=None,surface_sta
             #plt.figure();plt.imshow(np.max(surfaces[i_surf],axis=0))
     #print(f'!!! {surface_states}')
     #print(f'!!!{np.unique(labels)}')
-    border_dict=get_border_dict(labels,states=cell_states,nn_states=nn_states,vdist=vdist,radius=radius,scale=border_scale)
+    if state_ids is None:
+        state_ids=np.arange(1,max_cell_state+n_surf+1)
+    border_dict=get_border_dict(labels,states=cell_states,nn_states=state_ids,state_ids=state_ids,vdist=vdist,radius=radius,scale=border_scale)
     contact_properties=np.ones((max_cellid+1,np.max(cell_states)+1,1+n_moments))*np.nan
     border_properties_list=[None]*(1+n_moments)
     border_properties_list[0]='total'
